@@ -7,6 +7,7 @@ package cmdapi
 import (
 	"context"
 	"database/sql"
+	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
@@ -19,6 +20,7 @@ import (
 	"testing"
 	"time"
 
+	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	migrate2 "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
@@ -436,7 +438,12 @@ variable "url" {
 
 data "sql" "tenants" {
   url   = var.url
-  query = "SELECT name FROM tenants"
+  query = <<EOS
+SELECT name FROM tenants
+	WHERE mode LIKE ? AND active = ? AND created = ?
+EOS
+  # Pass all types of arguments.
+  args  = ["%test", true, 1]
 }
 
 env "local" {
@@ -466,7 +473,7 @@ env "local" {
 		require.NoError(t, err)
 		db, err := sql.Open("sqlite3", fmt.Sprintf("file:%s?cache=shared&_fk=1", filepath.Join(p, "tenants.db")))
 		require.NoError(t, err)
-		_, err = db.Exec("CREATE TABLE `tenants` (`name` TEXT)")
+		_, err = db.Exec("CREATE TABLE `tenants` (`name` TEXT, `mode` TEXT DEFAULT 'test', `active` BOOL DEFAULT TRUE, `created` INT DEFAULT 1);")
 		require.NoError(t, err)
 		_, err = db.Exec("INSERT INTO `tenants` (`name`) VALUES (?), (?), (?)", dbs[0], dbs[1], dbs[2])
 		require.NoError(t, err)
@@ -487,6 +494,27 @@ env "local" {
 			require.NoError(t, err)
 		}
 
+		cmd = migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err = runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--var", fmt.Sprintf("url=sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "tenants.db")),
+		)
+		require.NoError(t, err)
+		for _, s := range strings.Split(s, "\n") {
+			var r struct {
+				Tenant string
+				cmdlog.MigrateApply
+			}
+			require.NoError(t, json.Unmarshal([]byte(s), &r))
+			require.Empty(t, r.Pending)
+			require.Empty(t, r.Applied)
+			require.NotEmpty(t, r.Tenant)
+			require.Equal(t, "sqlite3", r.Driver)
+		}
+
 		_, err = db.Exec("INSERT INTO `tenants` (`name`) VALUES (NULL)")
 		require.NoError(t, err)
 		_, err = runCmd(
@@ -583,25 +611,81 @@ func TestMigrate_ApplyTxMode(t *testing.T) {
 }
 
 func TestMigrate_ApplyBaseline(t *testing.T) {
-	p := t.TempDir()
-	// Run migration with baseline should store this revision in the database.
-	s, err := runCmd(
-		migrateApplyCmd(),
-		"--dir", "file://testdata/baseline1",
-		"--baseline", "1",
-		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db")),
-	)
-	require.NoError(t, err)
-	require.Contains(t, s, "No migration files to execute")
+	t.Run("FromFlags", func(t *testing.T) {
+		p := t.TempDir()
+		// Run migration with baseline should store this revision in the database.
+		s, err := runCmd(
+			migrateApplyCmd(),
+			"--dir", "file://testdata/baseline1",
+			"--baseline", "1",
+			"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test1.db")),
+		)
+		require.NoError(t, err)
+		require.Contains(t, s, "No migration files to execute")
+		// Next run without baseline should run the migration from the baseline.
+		s, err = runCmd(
+			migrateApplyCmd(),
+			"--dir", "file://testdata/baseline1",
+			"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test1.db")),
+		)
+		require.NoError(t, err)
+		require.Contains(t, s, "No migration files to execute")
 
-	// Next run without baseline should run the migration from the baseline.
-	s, err = runCmd(
-		migrateApplyCmd(),
-		"--dir", "file://testdata/baseline2",
-		"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test.db")),
-	)
-	require.NoError(t, err)
-	require.Contains(t, s, "Migrating to version 20220318104615 from 1 (2 migrations in total)")
+		// Multiple migration files with baseline.
+		s, err = runCmd(
+			migrateApplyCmd(),
+			"--dir", "file://testdata/baseline2",
+			"--baseline", "1",
+			"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test2.db")),
+		)
+		require.NoError(t, err)
+		require.Contains(t, s, "Migrating to version 20220318104615 from 1 (2 migrations in total)")
+
+		// Run all migration files and skip baseline.
+		s, err = runCmd(
+			migrateApplyCmd(),
+			"--dir", "file://testdata/baseline2",
+			"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test3.db")),
+		)
+		require.NoError(t, err)
+		require.Contains(t, s, "Migrating to version 20220318104615 (3 migrations in total)")
+	})
+
+	t.Run("FromConfig", func(t *testing.T) {
+		const h = `
+env "local" {
+  migration {
+    baseline = "1"
+  }
+}`
+		p := t.TempDir()
+		path := filepath.Join(p, "atlas.hcl")
+		err := os.WriteFile(path, []byte(h), 0600)
+		require.NoError(t, err)
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--dir", "file://testdata/baseline1",
+			"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test1.db")),
+		)
+		require.NoError(t, err)
+		require.Contains(t, s, "No migration files to execute")
+
+		cmd = migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		s, err = runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--dir", "file://testdata/baseline2",
+			"--url", fmt.Sprintf("sqlite://file:%s?cache=shared&_fk=1", filepath.Join(p, "test2.db")),
+		)
+		require.NoError(t, err)
+		require.Contains(t, s, "Migrating to version 20220318104615 from 1 (2 migrations in total)")
+	})
 }
 
 func TestMigrate_Diff(t *testing.T) {
@@ -666,6 +750,46 @@ func TestMigrate_Diff(t *testing.T) {
 	)
 	require.True(t, strings.HasPrefix(s, "Error: acquiring database lock: "+errLock.Error()))
 	require.ErrorIs(t, err, errLock)
+
+	t.Run("Edit", func(t *testing.T) {
+		p := t.TempDir()
+		require.NoError(t, os.Setenv("EDITOR", "echo '-- Comment' >>"))
+		t.Cleanup(func() { require.NoError(t, os.Unsetenv("EDITOR")) })
+		args := []string{
+			"--edit",
+			"--dir", "file://" + p,
+			"--dev-url", openSQLite(t, ""),
+			"--to", to,
+		}
+		_, err := runCmd(migrateDiffCmd(), args...)
+		files, err := os.ReadDir(p)
+		require.NoError(t, err)
+		require.Len(t, files, 2)
+		b, err := os.ReadFile(filepath.Join(p, files[0].Name()))
+		require.NoError(t, err)
+		require.Contains(t, string(b), "CREATE")
+		require.True(t, strings.HasSuffix(string(b), "-- Comment\n"))
+		require.Equal(t, "atlas.sum", files[1].Name())
+
+		// Second run will have no effect.
+		_, err = runCmd(migrateDiffCmd(), args...)
+		require.NoError(t, err)
+		files, err = os.ReadDir(p)
+		require.NoError(t, err)
+		require.Len(t, files, 2)
+	})
+}
+
+func TestMigrate_StatusJSON(t *testing.T) {
+	p := t.TempDir()
+	s, err := runCmd(
+		migrateStatusCmd(),
+		"--dir", "file://"+p,
+		"-u", openSQLite(t, ""),
+		"--log", "{{ json .Env.Driver }}",
+	)
+	require.NoError(t, err)
+	require.Equal(t, `"sqlite3"`, s)
 }
 
 func TestMigrate_New(t *testing.T) {
@@ -731,6 +855,20 @@ func TestMigrate_New(t *testing.T) {
 	s, err = runCmd(migrateNewCmd(), "--dir", "file://testdata/mysql")
 	require.NotZero(t, s)
 	require.Error(t, err)
+
+	t.Run("Edit", func(t *testing.T) {
+		p := t.TempDir()
+		require.NoError(t, os.Setenv("EDITOR", "echo 'contents' >"))
+		t.Cleanup(func() { require.NoError(t, os.Unsetenv("EDITOR")) })
+		s, err = runCmd(migrateNewCmd(), "--dir", "file://"+p, "--edit")
+		files, err := os.ReadDir(p)
+		require.NoError(t, err)
+		require.Len(t, files, 2)
+		b, err := os.ReadFile(filepath.Join(p, files[0].Name()))
+		require.NoError(t, err)
+		require.Equal(t, "contents\n", string(b))
+		require.Equal(t, "atlas.sum", files[1].Name())
+	})
 }
 
 func TestMigrate_Validate(t *testing.T) {
