@@ -36,7 +36,7 @@ const (
 `
 )
 
-func TestCmdSchemaDiff(t *testing.T) {
+func TestSchema_Diff(t *testing.T) {
 	// Creates the missing table.
 	s, err := runCmd(
 		schemaDiffCmd(),
@@ -44,7 +44,7 @@ func TestCmdSchemaDiff(t *testing.T) {
 		"--to", openSQLite(t, "create table t1 (id int);"),
 	)
 	require.NoError(t, err)
-	require.EqualValues(t, "-- Create \"t1\" table\nCREATE TABLE `t1` (`id` int NULL)\n", s)
+	require.EqualValues(t, "-- Create \"t1\" table\nCREATE TABLE `t1` (`id` int NULL);\n", s)
 
 	// No changes.
 	s, err = runCmd(
@@ -71,7 +71,7 @@ func TestCmdSchemaDiff(t *testing.T) {
 		"--dev-url", openSQLite(t, ""),
 	)
 	require.NoError(t, err)
-	require.EqualValues(t, "-- Create \"tbl\" table\nCREATE TABLE `tbl` (`col` int NOT NULL, `col_2` bigint NULL)\n", s)
+	require.EqualValues(t, "-- Create \"tbl\" table\nCREATE TABLE `tbl` (`col` int NOT NULL, `col_2` bigint NULL);\n", s)
 
 	// Desired state from migration directory.
 	s, err = runCmd(
@@ -81,7 +81,7 @@ func TestCmdSchemaDiff(t *testing.T) {
 		"--dev-url", openSQLite(t, ""),
 	)
 	require.NoError(t, err)
-	require.EqualValues(t, "-- Create \"tbl\" table\nCREATE TABLE `tbl` (`col` int NOT NULL, `col_2` bigint NULL)\n", s)
+	require.EqualValues(t, "-- Create \"tbl\" table\nCREATE TABLE `tbl` (`col` int NOT NULL, `col_2` bigint NULL);\n", s)
 
 	// Current state from migration directory, desired state from HCL - synced.
 	p := filepath.Join(t.TempDir(), "schema.hcl")
@@ -130,7 +130,7 @@ table "tbl" {
 	require.NoError(t, err)
 	require.EqualValues(
 		t,
-		"-- Add column \"col_3\" to table: \"tbl\"\nALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL\n",
+		"-- Add column \"col_3\" to table: \"tbl\"\nALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL;\n",
 		s,
 	)
 
@@ -145,9 +145,9 @@ table "tbl" {
 	require.EqualValues(
 		t,
 		"-- Add column \"col_2\" to table: \"tbl\"\n"+
-			"ALTER TABLE `tbl` ADD COLUMN `col_2` bigint NULL\n"+
+			"ALTER TABLE `tbl` ADD COLUMN `col_2` bigint NULL;\n"+
 			"-- Add column \"col_3\" to table: \"tbl\"\n"+
-			"ALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL\n",
+			"ALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL;\n",
 		s,
 	)
 
@@ -180,7 +180,7 @@ table "tbl" {
 	require.NoError(t, err)
 	require.EqualValues(
 		t,
-		"-- Add column \"col_3\" to table: \"tbl\"\nALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL\n",
+		"-- Add column \"col_3\" to table: \"tbl\"\nALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL;\n",
 		s,
 	)
 	s, err = runCmd(
@@ -193,7 +193,7 @@ table "tbl" {
 	require.NoError(t, err)
 	require.EqualValues(
 		t,
-		"-- Add column \"col_3\" to table: \"tbl\"\nALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL\n",
+		"-- Add column \"col_3\" to table: \"tbl\"\nALTER TABLE `tbl` ADD COLUMN `col_3` text NOT NULL;\n",
 		s,
 	)
 
@@ -225,11 +225,11 @@ table "users" {
 			"--from", openSQLite(t, ""),
 		)
 		require.NoError(t, err)
-		require.Equal(t, "-- Create \"users\" table\nCREATE TABLE `users` (`id` int NOT NULL)\n", s)
+		require.Equal(t, "-- Create \"users\" table\nCREATE TABLE `users` (`id` int NOT NULL);\n", s)
 	})
 }
 
-func TestCmdSchemaApply(t *testing.T) {
+func TestSchema_Apply(t *testing.T) {
 	const drvName = "checknormalizer"
 	// If no dev-database is given, there must not be a call to Driver.Normalize.
 	sqlclient.Register(
@@ -412,7 +412,7 @@ func TestSchema_ApplyLog(t *testing.T) {
 	})
 }
 
-func TestCmdSchemaApply_Sources(t *testing.T) {
+func TesSchema_ApplySources(t *testing.T) {
 	var (
 		p   = t.TempDir()
 		cfg = filepath.Join(p, "atlas.hcl")
@@ -464,6 +464,100 @@ env "local" {
 	}, lines)
 }
 
+func TestSchema_ApplySQL(t *testing.T) {
+	t.Run("File", func(t *testing.T) {
+		db := openSQLite(t, "")
+		p := filepath.Join(t.TempDir(), "schema.sql")
+		require.NoError(t, os.WriteFile(p, []byte(`create table t1 (id int NOT NULL);`), 0600))
+		s, err := runCmd(
+			schemaApplyCmd(),
+			"-u", db,
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://"+p,
+			"--auto-approve",
+		)
+		require.NoError(t, err)
+		require.Equal(t, "-- Planned Changes:\n-- Create \"t1\" table\nCREATE TABLE `t1` (`id` int NOT NULL);\n", s)
+
+		s, err = runCmd(
+			schemaApplyCmd(),
+			"-u", db,
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://"+p,
+			"--auto-approve",
+		)
+		require.NoError(t, err)
+		require.Equal(t, "Schema is synced, no changes to be made.\n", s)
+	})
+	t.Run("Dir", func(t *testing.T) {
+		db := openSQLite(t, "")
+		s, err := runCmd(
+			schemaApplyCmd(),
+			"-u", db,
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://testdata/sqlite",
+			"--auto-approve",
+		)
+		require.NoError(t, err)
+		require.Equal(t, "-- Planned Changes:\n-- Create \"tbl\" table\nCREATE TABLE `tbl` (`col` int NOT NULL, `col_2` bigint NULL);\n", s)
+
+		s, err = runCmd(
+			schemaApplyCmd(),
+			"-u", db,
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://testdata/sqlite",
+			"--auto-approve",
+		)
+		require.NoError(t, err)
+		require.Equal(t, "Schema is synced, no changes to be made.\n", s)
+	})
+	t.Run("Error", func(t *testing.T) {
+		_, err := runCmd(
+			schemaApplyCmd(),
+			"-u", openSQLite(t, ""),
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://testdata/sqlite",
+			"--to", "file://testdata/sqlite2",
+			"--auto-approve",
+		)
+		require.EqualError(t, err, `the provided SQL state must be either a single schema file or a migration directory, but 2 paths were found`)
+
+		_, err = runCmd(
+			schemaApplyCmd(),
+			"-u", openSQLite(t, ""),
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://"+t.TempDir(),
+			"--auto-approve",
+		)
+		require.ErrorContains(t, err, `contains neither SQL nor HCL files`)
+
+		p := t.TempDir()
+		require.NoError(t, os.WriteFile(filepath.Join(p, "schema.sql"), []byte(`create table t1 (id int NOT NULL);`), 0600))
+		require.NoError(t, os.WriteFile(filepath.Join(p, "schema.hcl"), []byte(`schema "main" {}`), 0600))
+		_, err = runCmd(
+			schemaApplyCmd(),
+			"-u", openSQLite(t, ""),
+			"--dev-url", openSQLite(t, ""),
+			"--to", "file://"+p,
+			"--auto-approve",
+		)
+		require.EqualError(t, err, `ambiguous schema: both SQL and HCL files found: "schema.hcl", "schema.sql"`)
+	})
+}
+
+func TestSchema_InspectLog(t *testing.T) {
+	db := openSQLite(t, "create table t1 (id integer primary key);create table t2 (name text);")
+	cmd := schemaCmd()
+	cmd.AddCommand(schemaInspectCmd())
+	s, err := runCmd(
+		cmd, "inspect",
+		"-u", db,
+		"--log", "{{ json . }}",
+	)
+	require.NoError(t, err)
+	require.Equal(t, `{"schemas":[{"name":"main","tables":[{"name":"t1","columns":[{"name":"id","type":"INTEGER","null":true}],"primary_key":{"parts":[{"column":"id"}]}},{"name":"t2","columns":[{"name":"name","type":"TEXT","null":true}]}]}]}`, s)
+}
+
 func TestFmt(t *testing.T) {
 	for _, tt := range []struct {
 		name          string
