@@ -16,8 +16,8 @@ function Nav() {
             </Link>
         </li>
         <li className={index.linkItem}>
-            <Link to="/knowledge">
-                Knowledge Center
+            <Link to="/guides">
+                Guides
             </Link>
         </li>
         <li className={index.linkItem}>
@@ -80,7 +80,7 @@ export default function () {
                     <AtlasButton style={{"marginTop": "5%"}} text="Get Started" link="/getting-started" type="primaryButton"/>
                 </div>
                 <div className={index.imageContainer}>
-                    <img src="https://atlasgo.io/uploads/hero2.png" alt="hero"/>
+                    <img src="https://atlasgo.io/uploads/images/atlas-hero-v1.png" alt="hero"/>
                 </div>
             </div>
         </div>
@@ -101,7 +101,7 @@ export default function () {
                 </section>
 
                 <button className={index.textButton}>
-                    <Link to="/ddl/intro">Read More <span>&#8594;</span></Link>
+                    <Link to="/atlas-schema/sql-resources">Read More <span>&#8594;</span></Link>
                 </button>
 
                 <BrowserOnly>
