@@ -8,6 +8,7 @@ import (
 	"bytes"
 	"context"
 	"database/sql"
+	"encoding/json"
 	"fmt"
 	"os"
 	"os/exec"
@@ -45,6 +46,7 @@ func TestMySQL_Script(t *testing.T) {
 				"execsql":     t.cmdExec,
 				"atlas":       t.cmdCLI,
 				"clearSchema": t.clearSchema,
+				"validJSON":   validJSON,
 			},
 		})
 	})
@@ -96,7 +98,7 @@ func (t *myTest) setupScript(env *testscript.Env) error {
 	env.Setenv("version", t.version)
 	env.Setenv("charset", attrs[0].(*schema.Charset).V)
 	env.Setenv("collate", attrs[1].(*schema.Collation).V)
-	if err := replaceDBURL(env, t.dsn("")); err != nil {
+	if err := replaceDBURL(env, t.url("")); err != nil {
 		return err
 	}
 	return setupScript(t.T, env, t.db, "DROP SCHEMA IF EXISTS %s")
@@ -114,7 +116,7 @@ func replaceDBURL(env *testscript.Env, url string) error {
 
 func (t *pgTest) setupScript(env *testscript.Env) error {
 	env.Setenv("version", t.version)
-	u := strings.ReplaceAll(t.dsn(""), "/test", "/")
+	u := strings.ReplaceAll(t.url(""), "/test", "/")
 	if err := replaceDBURL(env, u); err != nil {
 		return err
 	}
@@ -369,11 +371,11 @@ func (t *liteTest) cmdExec(ts *testscript.TestScript, _ bool, args []string) {
 }
 
 func (t *myTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
-	cmdCLI(ts, neg, args, t.dsn(ts.Getenv("db")), ts.Getenv(atlasPathKey))
+	cmdCLI(ts, neg, args, t.url(ts.Getenv("db")), ts.Getenv(atlasPathKey))
 }
 
 func (t *pgTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
-	cmdCLI(ts, neg, args, t.dsn(ts.Getenv("db")), ts.Getenv(atlasPathKey))
+	cmdCLI(ts, neg, args, t.url(ts.Getenv("db")), ts.Getenv(atlasPathKey))
 }
 
 func (t *liteTest) cmdCLI(ts *testscript.TestScript, neg bool, args []string) {
@@ -705,3 +707,7 @@ func matchErr(ts *testscript.TestScript, err error, p string) {
 		ts.Fatalf("mismatched errors: %v != %s", err, p)
 	}
 }
+
+func validJSON(ts *testscript.TestScript, _ bool, args []string) {
+	ts.Check(json.Unmarshal([]byte(ts.ReadFile(args[0])), new(map[string]any)))
+}
