@@ -317,18 +317,18 @@ func TestSchemaRefParse(t *testing.T) {
 		Points []*Point `spec:"point"`
 	}{
 		Points: []*Point{
-			{Z: []*Ref{{V: "$a"}}},
-			{Z: []*Ref{{V: "b"}}},
+			{Z: []*Ref{{V: "$point.1"}}},
+			{Z: []*Ref{{V: "$point.0"}}},
 		},
 	}
 	b, err := Marshal(&test)
 	require.NoError(t, err)
 	expected :=
 		`point {
-  z = [a]
+  z = [point.1]
 }
 point {
-  z = [b]
+  z = [point.0]
 }
 `
 	require.Equal(t, expected, string(b))
@@ -472,6 +472,44 @@ r = user.atlas.cli
 	require.EqualValues(t, "$user.atlas.cli", test.R.V)
 }
 
+func TestQuotedRefs(t *testing.T) {
+	h := `
+user "ariel.mashraki" {
+  username = "a8m"
+}
+account "ariga.cloud" "a8m.dev" {
+  org = "dev"
+}
+env "dev" "a8m.dev" {}
+env "prod.1" "a8m" {}
+
+v = user["ariel.mashraki"].username
+r = user["ariel.mashraki"]
+vs = [account["ariga.cloud"]["a8m.dev"].org]
+rs = [
+	account["ariga.cloud"]["a8m.dev"],
+	env["dev"]["a8m.dev"],
+	env["prod.1"]["a8m"],
+]
+`
+	var test struct {
+		V  string   `spec:"v"`
+		R  *Ref     `spec:"r"`
+		Vs []string `spec:"vs"`
+		Rs []*Ref   `spec:"rs"`
+	}
+	err := New().EvalBytes([]byte(h), &test, nil)
+	require.NoError(t, err)
+	require.EqualValues(t, "a8m", test.V)
+	require.EqualValues(t, `$user["ariel.mashraki"]`, test.R.V)
+	require.EqualValues(t, []string{"dev"}, test.Vs)
+	require.EqualValues(t, []*Ref{
+		{`$account["ariga.cloud"]["a8m.dev"]`},
+		{`$env.dev["a8m.dev"]`},
+		{`$env["prod.1"].a8m`},
+	}, test.Rs)
+}
+
 func TestInputValues(t *testing.T) {
 	h := `
 variable "name" {
@@ -502,6 +540,7 @@ variable "convert_bool" {
 variable "strings" {
   type = list(string)
   default = ["a", "b"]
+  description = "description is a valid attribute"
 }
 
 name = var.name
@@ -512,7 +551,6 @@ convert_int = var.convert_int
 convert_bool = var.convert_bool
 strings = var.strings
 `
-	state := New()
 	var test struct {
 		Name        string   `spec:"name"`
 		Default     string   `spec:"default"`
@@ -522,7 +560,7 @@ strings = var.strings
 		ConvertBool bool     `spec:"convert_bool"`
 		Strings     []string `spec:"strings"`
 	}
-	err := state.EvalBytes([]byte(h), &test, map[string]cty.Value{
+	err := New().EvalBytes([]byte(h), &test, map[string]cty.Value{
 		"name":         cty.StringVal("rotemtam"),
 		"int":          cty.NumberIntVal(42),
 		"bool":         cty.BoolVal(true),
@@ -539,3 +577,21 @@ strings = var.strings
 	require.EqualValues(t, true, test.ConvertBool)
 	require.EqualValues(t, []string{"a", "b"}, test.Strings)
 }
+
+func TestVariable_InvalidType(t *testing.T) {
+	h := `
+variable "name" {
+  type = "int"
+  default = "boring"
+}`
+	err := New().EvalBytes([]byte(h), &struct{}{}, nil)
+	require.EqualError(t, err, `invalid type "int" for variable "name". Valid types are: string, number, bool, list, map, or set`)
+
+	h = `
+variable "name" {
+  type = "boring"
+  default = "boring"
+}`
+	err = New().EvalBytes([]byte(h), &struct{}{}, nil)
+	require.EqualError(t, err, `invalid type "boring" for variable "name". Valid types are: string, number, bool, list, map, or set`)
+}
