@@ -56,6 +56,40 @@ func ApplyChanges(ctx context.Context, changes []schema.Change, p execPlanner, o
 	return nil
 }
 
+// noRows implements the schema.ExecQuerier for migrate.Driver's without connections.
+// This can be useful to always return no rows for queries, and block any execution.
+type noRows struct{}
+
+// QueryContext implements the sqlx.ExecQuerier interface.
+func (*noRows) QueryContext(context.Context, string, ...interface{}) (*sql.Rows, error) {
+	return nil, sql.ErrNoRows
+}
+
+// ExecContext implements the sqlx.ExecQuerier interface.
+func (*noRows) ExecContext(context.Context, string, ...interface{}) (sql.Result, error) {
+	return nil, errors.New("cannot execute statements without a database connection. use Open to create a new Driver")
+}
+
+// NoRows to be used by differs and planners without a connection.
+var NoRows schema.ExecQuerier = (*noRows)(nil)
+
+// SetReversible sets the Reversible field to
+// true if all planned changes are reversible.
+func SetReversible(p *migrate.Plan) error {
+	reversible := true
+	for _, c := range p.Changes {
+		stmts, err := c.ReverseStmts()
+		if err != nil {
+			return err
+		}
+		if len(stmts) == 0 {
+			reversible = false
+		}
+	}
+	p.Reversible = reversible
+	return nil
+}
+
 // DetachCycles takes a list of schema changes, and detaches
 // references between changes if there is at least one circular
 // reference in the changeset. More explicitly, it postpones fks
