@@ -54,6 +54,8 @@ type (
 		Bytes() []byte
 		// Stmts returns the set of SQL statements this file holds.
 		Stmts() ([]string, error)
+		// StmtDecls returns the set of SQL statements this file holds alongside its preceding comments.
+		StmtDecls() ([]*Stmt, error)
 	}
 )
 
@@ -115,28 +117,11 @@ func (d *LocalDir) Files() ([]File, error) {
 
 // Checksum implements Dir.Checksum. By default, it calls Files() and creates a checksum from them.
 func (d *LocalDir) Checksum() (HashFile, error) {
-	var (
-		hs HashFile
-		h  = sha256.New()
-	)
 	files, err := d.Files()
 	if err != nil {
 		return nil, err
 	}
-	for _, f := range files {
-		if _, err = h.Write([]byte(f.Name())); err != nil {
-			return nil, err
-		}
-		// Check if this file contains an "atlas:sum" directive and if so, act to it.
-		if mode, ok := directive(string(f.Bytes()), directiveSum); ok && mode == sumModeIgnore {
-			continue
-		}
-		if _, err = h.Write(f.Bytes()); err != nil {
-			return nil, err
-		}
-		hs = append(hs, struct{ N, H string }{f.Name(), base64.StdEncoding.EncodeToString(h.Sum(nil))})
-	}
-	return hs, nil
+	return NewHashFile(files)
 }
 
 // LocalFile is used by LocalDir to implement the Scanner interface.
@@ -171,9 +156,22 @@ func (f LocalFile) Version() string {
 	return strings.SplitN(strings.TrimSuffix(f.n, ".sql"), "_", 2)[0]
 }
 
-// Stmts implements Scanner.Stmts. It reads migration file line-by-line and expects a statement to be one line only.
+// Stmts returns the SQL statement exists in the local file.
 func (f LocalFile) Stmts() ([]string, error) {
-	return stmts(string(f.b))
+	s, err := Stmts(string(f.b))
+	if err != nil {
+		return nil, err
+	}
+	stmts := make([]string, len(s))
+	for i := range s {
+		stmts[i] = s[i].Text
+	}
+	return stmts, nil
+}
+
+// StmtDecls returns the all statement declarations exist in the local file.
+func (f LocalFile) StmtDecls() ([]*Stmt, error) {
+	return Stmts(string(f.b))
 }
 
 // Bytes returns local file data.
@@ -181,6 +179,65 @@ func (f LocalFile) Bytes() []byte {
 	return f.b
 }
 
+// MemDir provides an in-memory Dir implementation.
+type MemDir struct {
+	files map[string]File
+}
+
+// Open implements fs.FS.
+func (d *MemDir) Open(name string) (fs.File, error) {
+	var b []byte
+	switch name {
+	case HashFileName:
+		h, err := d.Checksum()
+		if err != nil {
+			return nil, err
+		}
+		if b, err = h.MarshalText(); err != nil {
+			return nil, err
+		}
+	default:
+		f, ok := d.files[name]
+		if !ok {
+			return nil, fs.ErrNotExist
+		}
+		b = f.Bytes()
+	}
+	return &memFile{
+		ReadCloser: io.NopCloser(bytes.NewReader(b)),
+	}, nil
+}
+
+// WriteFile adds a new file in-memory.
+func (d *MemDir) WriteFile(name string, data []byte) error {
+	if d.files == nil {
+		d.files = make(map[string]File)
+	}
+	d.files[name] = NewLocalFile(name, data)
+	return nil
+}
+
+// Files returns a set of files stored in-memory to be executed on a database.
+func (d *MemDir) Files() ([]File, error) {
+	files := make([]File, 0, len(d.files))
+	for _, f := range d.files {
+		files = append(files, f)
+	}
+	sort.Slice(files, func(i, j int) bool {
+		return files[i].Name() < files[j].Name()
+	})
+	return files, nil
+}
+
+// Checksum implements Dir.Checksum.
+func (d *MemDir) Checksum() (HashFile, error) {
+	files, err := d.Files()
+	if err != nil {
+		return nil, err
+	}
+	return NewHashFile(files)
+}
+
 var (
 	// templateFuncs contains the template.FuncMap for the DefaultFormatter.
 	templateFuncs = template.FuncMap{"now": func() string { return time.Now().UTC().Format("20060102150405") }}
@@ -189,9 +246,9 @@ var (
 		templates: []struct{ N, C *template.Template }{
 			{
 				N: template.Must(template.New("").Funcs(templateFuncs).Parse(
-					"{{ now }}{{ with .Name }}_{{ . }}{{ end }}.sql",
+					"{{ with .Version }}{{ . }}{{ else }}{{ now }}{{ end }}{{ with .Name }}_{{ . }}{{ end }}.sql",
 				)),
-				C: template.Must(template.New("").Funcs(templateFuncs).Parse(
+				C: template.Must(template.New("").Parse(
 					`{{ range .Changes }}{{ with .Comment }}-- {{ println . }}{{ end }}{{ printf "%s;\n" .Cmd }}{{ end }}`,
 				)),
 			},
@@ -246,11 +303,26 @@ const HashFileName = "atlas.sum"
 // HashFile represents the integrity sum file of the migration dir.
 type HashFile []struct{ N, H string }
 
-// HashSum reads the whole dir, sorts the files by name and creates a HashSum from its contents.
-//
-// Deprecated: Use Dir.Checksum instead.
-func HashSum(dir Dir) (HashFile, error) {
-	return dir.Checksum()
+// NewHashFile computes and returns a HashFile from the given directory's files.
+func NewHashFile(files []File) (HashFile, error) {
+	var (
+		hs HashFile
+		h  = sha256.New()
+	)
+	for _, f := range files {
+		if _, err := h.Write([]byte(f.Name())); err != nil {
+			return nil, err
+		}
+		// Check if this file contains an "atlas:sum" directive and if so, act to it.
+		if mode, ok := directive(string(f.Bytes()), directiveSum); ok && mode == sumModeIgnore {
+			continue
+		}
+		if _, err := h.Write(f.Bytes()); err != nil {
+			return nil, err
+		}
+		hs = append(hs, struct{ N, H string }{f.Name(), base64.StdEncoding.EncodeToString(h.Sum(nil))})
+	}
+	return hs, nil
 }
 
 // WriteSumFile writes the given HashFile to the Dir. If the file does not exist, it is created.
@@ -300,7 +372,8 @@ func (f *HashFile) UnmarshalText(b []byte) error {
 	return sc.Err()
 }
 
-func (f HashFile) sumByName(n string) (string, error) {
+// SumByName returns the hash for a migration file by its name.
+func (f HashFile) SumByName(n string) (string, error) {
 	for _, f := range f {
 		if f.N == n {
 			return f.H, nil
@@ -397,3 +470,15 @@ func readHashFile(dir Dir) (HashFile, error) {
 	}
 	return fh, nil
 }
+
+// memFile implements the File interface for a file in memory.
+type memFile struct{ io.ReadCloser }
+
+// Stat returns a zero FileInfo.
+func (m *memFile) Stat() (fs.FileInfo, error) { return m, nil }
+func (m *memFile) Name() string               { return "" }
+func (m *memFile) Size() int64                { return 0 }
+func (m *memFile) Mode() fs.FileMode          { return 0 }
+func (m *memFile) ModTime() time.Time         { return time.Time{} }
+func (m *memFile) IsDir() bool                { return false }
+func (m *memFile) Sys() interface{}           { return nil }
