@@ -21,6 +21,31 @@ import (
 	"github.com/stretchr/testify/require"
 )
 
+func TestRevisionType_MarshalText(t *testing.T) {
+	for _, tt := range []struct {
+		r       migrate.RevisionType
+		ex      string
+		wantErr bool
+	}{
+		{migrate.RevisionTypeUnknown, "", true},
+		{migrate.RevisionTypeBaseline, "baseline", false},
+		{migrate.RevisionTypeExecute, "applied", false},
+		{migrate.RevisionTypeResolved, "manually set", false},
+		{migrate.RevisionTypeExecute | migrate.RevisionTypeResolved, "applied + manually set", false},
+		{migrate.RevisionTypeExecute | migrate.RevisionTypeBaseline, "", true},
+		{1 << 3, "", true},
+	} {
+		ac, err := tt.r.MarshalText()
+		if tt.wantErr {
+			require.Error(t, err)
+			require.Zero(t, ac)
+			continue
+		}
+		require.NoError(t, err)
+		require.Equal(t, tt.ex, string(ac))
+	}
+}
+
 func TestPlanner_WritePlan(t *testing.T) {
 	p := t.TempDir()
 	d, err := migrate.NewLocalDir(p)
@@ -122,17 +147,43 @@ func TestPlanner_PlanSchema(t *testing.T) {
 
 func TestExecutor_Replay(t *testing.T) {
 	ctx := context.Background()
-	d, err := migrate.NewLocalDir("testdata/migrate")
+	d, err := migrate.NewLocalDir(filepath.FromSlash("testdata/migrate"))
 	require.NoError(t, err)
 
 	drv := &mockDriver{}
 	ex, err := migrate.NewExecutor(drv, d, migrate.NopRevisionReadWriter{})
 	require.NoError(t, err)
 
+	// No options.
 	_, err = ex.Replay(ctx, migrate.RealmConn(drv, nil))
 	require.NoError(t, err)
 	require.Equal(t, []string{"DROP TABLE IF EXISTS t;", "CREATE TABLE t(c int);"}, drv.executed)
 
+	// ReplayVersion option.
+	*drv = mockDriver{}
+	dir, err := migrate.NewLocalDir(filepath.FromSlash("testdata/migrate/sub"))
+	require.NoError(t, err)
+	*d = *dir
+	_, err = ex.Replay(ctx, migrate.RealmConn(drv, nil), migrate.ReplayToVersion("0"))
+	require.ErrorContains(t, err, "migration with version \"0\" not found")
+	require.Zero(t, drv.executed)
+
+	*drv = mockDriver{}
+	_, err = ex.Replay(ctx, migrate.RealmConn(drv, nil), migrate.ReplayToVersion("1.a"))
+	require.NoError(t, err)
+	require.Equal(t, []string{"CREATE TABLE t_sub(c int);", "ALTER TABLE t_sub ADD c1 int;"}, drv.executed)
+
+	*drv = mockDriver{}
+	_, err = ex.Replay(ctx, migrate.RealmConn(drv, nil), migrate.ReplayToVersion("3"))
+	require.NoError(t, err)
+	require.Equal(t, []string{
+		"CREATE TABLE t_sub(c int);",
+		"ALTER TABLE t_sub ADD c1 int;",
+		"ALTER TABLE t_sub ADD c2 int;",
+		"ALTER TABLE t_sub ADD c3 int;",
+		"ALTER TABLE t_sub ADD c4 int;",
+	}, drv.executed)
+
 	// Does not work if database is not clean.
 	drv.dirty = true
 	drv.realm = schema.Realm{Schemas: []*schema.Schema{{Name: "schema"}}}
@@ -195,6 +246,53 @@ func TestExecutor_Pending(t *testing.T) {
 	p, err = ex.Pending(context.Background())
 	require.NoError(t, err)
 	require.Len(t, p, 1)
+
+	// Nothing to do if all migrations are applied.
+	rev3.Applied = rev3.Total
+	*rrw = []*migrate.Revision{rev1, rev2, rev3}
+	p, err = ex.Pending(context.Background())
+	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+	require.Len(t, p, 0)
+
+	// If there is a revision in the past with no existing migration, we don't care.
+	rev3.Applied = rev3.Total
+	*rrw = []*migrate.Revision{{Version: "2.11"}, rev3}
+	p, err = ex.Pending(context.Background())
+	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+	require.Len(t, p, 0)
+
+	// If only the last migration file is applied, we expect there are no pending files.
+	*rrw = []*migrate.Revision{rev3}
+	p, err = ex.Pending(context.Background())
+	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+	require.Len(t, p, 0)
+
+	// If the last applied revision has no matching migration file, and the last
+	// migration version precedes that revision, there is nothing to do.
+	*rrw = []*migrate.Revision{{Version: "5"}}
+	p, err = ex.Pending(context.Background())
+	require.ErrorIs(t, err, migrate.ErrNoPendingFiles)
+	require.Len(t, p, 0)
+
+	// The applied revision precedes every migration file. Expect all files pending.
+	*rrw = []*migrate.Revision{{Version: "1.1"}}
+	p, err = ex.Pending(context.Background())
+	require.NoError(t, err)
+	require.Len(t, p, 3)
+
+	// All except one file are applied. The latest revision does not exist and its version is smaller than the last
+	// migration file. Expect the last file pending.
+	*rrw = []*migrate.Revision{rev1, rev2, {Version: "2.11"}}
+	p, err = ex.Pending(context.Background())
+	require.NoError(t, err)
+	require.Len(t, p, 1)
+	require.Equal(t, rev3.Version, p[0].Version())
+
+	// If the last revision is partially applied and the matching migration file does not exist, we have a problem.
+	*rrw = []*migrate.Revision{{Version: "deleted", Description: "desc", Total: 1}}
+	p, err = ex.Pending(context.Background())
+	require.EqualError(t, err, migrate.MissingMigrationError{Version: "deleted", Description: "desc"}.Error())
+	require.Len(t, p, 0)
 }
 
 func TestExecutor(t *testing.T) {
@@ -221,6 +319,7 @@ func TestExecutor(t *testing.T) {
 	require.NoError(t, err)
 	require.NotNil(t, ex)
 	require.ErrorIs(t, ex.ExecuteN(context.Background(), 0), migrate.ErrChecksumMismatch)
+	require.ErrorIs(t, ex.ExecuteTo(context.Background(), ""), migrate.ErrChecksumMismatch)
 
 	// Prerequisites.
 	var (
@@ -246,7 +345,7 @@ func TestExecutor(t *testing.T) {
 			OperatorVersion: "op",
 		}
 	)
-	dir, err = migrate.NewLocalDir(filepath.Join("testdata/migrate", "sub"))
+	dir, err = migrate.NewLocalDir(filepath.Join("testdata", "migrate", "sub"))
 	require.NoError(t, err)
 	ex, err = migrate.NewExecutor(drv, dir, rrw, migrate.WithLogger(log), migrate.WithOperatorVersion("op"))
 	require.NoError(t, err)
@@ -257,15 +356,18 @@ func TestExecutor(t *testing.T) {
 		"CREATE TABLE t_sub(c int);", "ALTER TABLE t_sub ADD c1 int;", "ALTER TABLE t_sub ADD c2 int;",
 	})
 	requireEqualRevisions(t, []*migrate.Revision{rev1, rev2}, *rrw)
-	require.Equal(t, []migrate.LogEntry{
-		migrate.LogExecution{To: "2.10.x-20", Files: []string{"1.a_sub.up.sql", "2.10.x-20_description.sql"}},
-		migrate.LogFile{Version: "1.a", Desc: "sub.up"},
-		migrate.LogStmt{SQL: "CREATE TABLE t_sub(c int);"},
-		migrate.LogStmt{SQL: "ALTER TABLE t_sub ADD c1 int;"},
-		migrate.LogFile{Version: "2.10.x-20", Desc: "description"},
-		migrate.LogStmt{SQL: "ALTER TABLE t_sub ADD c2 int;"},
-		migrate.LogDone{},
-	}, []migrate.LogEntry(*log))
+	require.Len(t, *log, 7)
+	require.IsType(t, migrate.LogExecution{}, (*log)[0])
+	require.Equal(t, "2.10.x-20", (*log)[0].(migrate.LogExecution).To)
+	require.Len(t, (*log)[0].(migrate.LogExecution).Files, 2)
+	require.Equal(t, "1.a_sub.up.sql", (*log)[0].(migrate.LogExecution).Files[0].Name())
+	require.Equal(t, "2.10.x-20_description.sql", (*log)[0].(migrate.LogExecution).Files[1].Name())
+	require.IsType(t, migrate.LogFile{}, (*log)[1])
+	require.Equal(t, migrate.LogStmt{SQL: "CREATE TABLE t_sub(c int);"}, (*log)[2])
+	require.Equal(t, migrate.LogStmt{SQL: "ALTER TABLE t_sub ADD c1 int;"}, (*log)[3])
+	require.IsType(t, migrate.LogFile{}, (*log)[4])
+	require.Equal(t, migrate.LogStmt{SQL: "ALTER TABLE t_sub ADD c2 int;"}, (*log)[5])
+	require.Equal(t, migrate.LogDone{}, (*log)[6])
 
 	// Partly is pending.
 	p, err := ex.Pending(context.Background())
@@ -320,7 +422,8 @@ func TestExecutor(t *testing.T) {
 		Type:            migrate.RevisionTypeExecute,
 		Applied:         1,
 		Total:           2,
-		Error:           "Statement:\nALTER TABLE t_sub ADD c4 int;\n\nError:\nthis is an error",
+		Error:           "this is an error",
+		ErrorStmt:       "ALTER TABLE t_sub ADD c4 int;",
 		OperatorVersion: "op",
 	}, revs[len(revs)-1])
 
@@ -337,6 +440,13 @@ func TestExecutor(t *testing.T) {
 
 	// Everything is applied.
 	require.ErrorIs(t, ex.ExecuteN(context.Background(), 0), migrate.ErrNoPendingFiles)
+
+	// Test ExecuteTo.
+	*rrw = []*migrate.Revision{}
+	*drv = mockDriver{}
+	require.EqualError(t, ex.ExecuteTo(context.Background(), ""), "sql/migrate: execute: migration with version \"\" not found")
+	require.NoError(t, ex.ExecuteTo(context.Background(), "2.10.x-20"))
+	requireEqualRevisions(t, []*migrate.Revision{rev1, rev2}, *rrw)
 }
 
 func TestExecutor_Baseline(t *testing.T) {
@@ -524,7 +634,7 @@ func (rrw *mockRevisionReadWriter) WriteRevision(_ context.Context, r *migrate.R
 }
 
 func (rrw *mockRevisionReadWriter) ReadRevision(_ context.Context, v string) (*migrate.Revision, error) {
-	for _, r := range []*migrate.Revision(*rrw) {
+	for _, r := range *rrw {
 		if r.Version == v {
 			return r, nil
 		}
@@ -532,6 +642,22 @@ func (rrw *mockRevisionReadWriter) ReadRevision(_ context.Context, v string) (*m
 	return nil, migrate.ErrRevisionNotExist
 }
 
+func (rrw *mockRevisionReadWriter) DeleteRevision(_ context.Context, v string) error {
+	i := -1
+	for j, r := range *rrw {
+		if r.Version == v {
+			i = j
+			break
+		}
+	}
+	if i == -1 {
+		return nil
+	}
+	copy((*rrw)[i:], (*rrw)[i+1:])
+	*rrw = (*rrw)[:len(*rrw)-1]
+	return nil
+}
+
 func (rrw *mockRevisionReadWriter) ReadRevisions(context.Context) ([]*migrate.Revision, error) {
 	return *rrw, nil
 }
