@@ -41,8 +41,24 @@ func TestMigrate_ApplyChanges(t *testing.T) {
 	err = migrate.ApplyChanges(context.Background(), []schema.Change{
 		&schema.AddSchema{S: &schema.Schema{Name: "test", Attrs: []schema.Attr{&schema.Charset{V: "latin"}}}},
 		&schema.DropSchema{S: &schema.Schema{Name: "atlas", Attrs: []schema.Attr{&schema.Charset{V: "latin"}}}},
-		&schema.DropTable{T: &schema.Table{Name: "users"}},
-		&schema.DropTable{T: &schema.Table{Name: "pets", Schema: &schema.Schema{Name: "public"}}, Extra: []schema.Clause{&schema.IfExists{}}},
+		&schema.DropTable{
+			T: &schema.Table{
+				Name: "users",
+				Columns: []*schema.Column{
+					{Name: "id", Type: &schema.ColumnType{Raw: "bigint", Type: &schema.IntegerType{T: "bigint"}}},
+				},
+			},
+		},
+		&schema.DropTable{
+			T: &schema.Table{
+				Name:   "pets",
+				Schema: &schema.Schema{Name: "public"},
+				Columns: []*schema.Column{
+					{Name: "id", Type: &schema.ColumnType{Raw: "bigint", Type: &schema.IntegerType{T: "bigint"}}},
+				},
+			},
+			Extra: []schema.Clause{&schema.IfExists{}},
+		},
 		&schema.AddTable{
 			T: func() *schema.Table {
 				t := &schema.Table{
@@ -175,9 +191,24 @@ func TestPlanChanges(t *testing.T) {
 	tests := []struct {
 		version  string
 		changes  []schema.Change
+		options  []migrate.PlanOption
 		wantPlan *migrate.Plan
 		wantErr  bool
 	}{
+		{
+			changes: []schema.Change{
+				&schema.AddTable{T: schema.NewTable("users")},
+			},
+			// Table "users" has no columns.
+			wantErr: true,
+		},
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{T: schema.NewTable("users")},
+			},
+			// Table "users" has no columns; drop the table instead.
+			wantErr: true,
+		},
 		{
 			changes: []schema.Change{
 				&schema.AddSchema{S: schema.New("test").SetCharset("utf8mb4"), Extra: []schema.Clause{&schema.IfNotExists{}}},
@@ -432,10 +463,16 @@ func TestPlanChanges(t *testing.T) {
 		},
 		{
 			changes: []schema.Change{
-				&schema.DropTable{T: &schema.Table{Name: "posts"}},
+				&schema.DropTable{T: schema.NewTable("posts").AddColumns(schema.NewIntColumn("id", "bigint"))},
 			},
 			wantPlan: &migrate.Plan{
-				Changes: []*migrate.Change{{Cmd: "DROP TABLE `posts`"}},
+				Reversible: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     "DROP TABLE `posts`",
+						Reverse: "CREATE TABLE `posts` (`id` bigint NOT NULL)",
+					},
+				},
 			},
 		},
 		{
@@ -664,7 +701,8 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("users"),
+					T: schema.NewTable("users").
+						AddColumns(schema.NewColumn("c").SetGeneratedExpr(&schema.GeneratedExpr{Expr: "1"})),
 					Changes: []schema.Change{
 						&schema.ModifyColumn{
 							Change: schema.ChangeGenerated,
@@ -680,7 +718,8 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("users"),
+					T: schema.NewTable("users").
+						AddColumns(schema.NewColumn("c")),
 					Changes: []schema.Change{
 						&schema.ModifyColumn{
 							Change: schema.ChangeGenerated,
@@ -696,7 +735,8 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("users"),
+					T: schema.NewTable("users").
+						AddColumns(schema.NewColumn("c").SetGeneratedExpr(&schema.GeneratedExpr{Expr: "1", Type: "STORED"})),
 					Changes: []schema.Change{
 						&schema.ModifyColumn{
 							Change: schema.ChangeGenerated,
@@ -712,7 +752,8 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("users"),
+					T: schema.NewTable("users").
+						AddColumns(schema.NewIntColumn("c", "int")),
 					Changes: []schema.Change{
 						&schema.ModifyColumn{
 							Change: schema.ChangeGenerated,
@@ -736,7 +777,8 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("users"),
+					T: schema.NewTable("users").
+						AddColumns(schema.NewIntColumn("c", "int").SetGeneratedExpr(&schema.GeneratedExpr{Expr: "1", Type: "STORED"})),
 					Changes: []schema.Change{
 						&schema.ModifyColumn{
 							Change: schema.ChangeGenerated,
@@ -793,7 +835,9 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("t1").SetSchema(schema.New("s1")),
+					T: schema.NewTable("t1").
+						SetSchema(schema.New("s1")).
+						AddColumns(schema.NewColumn("b")),
 					Changes: []schema.Change{
 						&schema.RenameColumn{
 							From: schema.NewColumn("a"),
@@ -816,7 +860,9 @@ func TestPlanChanges(t *testing.T) {
 			version: "5.6",
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("t1").SetSchema(schema.New("s1")),
+					T: schema.NewTable("t1").
+						SetSchema(schema.New("s1")).
+						AddColumns(schema.NewIntColumn("b", "int")),
 					Changes: []schema.Change{
 						&schema.RenameColumn{
 							From: schema.NewIntColumn("a", "int"),
@@ -838,7 +884,9 @@ func TestPlanChanges(t *testing.T) {
 		{
 			changes: []schema.Change{
 				&schema.ModifyTable{
-					T: schema.NewTable("t1").SetSchema(schema.New("s1")),
+					T: schema.NewTable("t1").
+						SetSchema(schema.New("s1")).
+						AddColumns(schema.NewIntColumn("b", "int")),
 					Changes: []schema.Change{
 						&schema.RenameIndex{
 							From: schema.NewIndex("a"),
@@ -857,6 +905,56 @@ func TestPlanChanges(t *testing.T) {
 				},
 			},
 		},
+		// Empty qualifier.
+		{
+			changes: []schema.Change{
+				&schema.AddTable{T: schema.NewTable("t").SetSchema(schema.New("d")).AddColumns(schema.NewIntColumn("a", "int"))},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
+			wantPlan: &migrate.Plan{
+				Reversible: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     "CREATE TABLE `t` (`a` int NOT NULL)",
+						Reverse: "DROP TABLE `t`",
+					},
+				},
+			},
+		},
+		// Custom qualifier.
+		{
+			changes: []schema.Change{
+				&schema.AddTable{T: schema.NewTable("t").SetSchema(schema.New("d")).AddColumns(schema.NewIntColumn("a", "int"))},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) {
+					s := "other"
+					o.SchemaQualifier = &s
+				},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     "CREATE TABLE `other`.`t` (`a` int NOT NULL)",
+						Reverse: "DROP TABLE `other`.`t`",
+					},
+				},
+			},
+		},
+		// Empty qualifier in multi-schema mode should fail.
+		{
+			changes: []schema.Change{
+				&schema.AddTable{T: schema.NewTable("t1").SetSchema(schema.New("s1")).AddColumns(schema.NewIntColumn("a", "int"))},
+				&schema.AddTable{T: schema.NewTable("t2").SetSchema(schema.New("s2")).AddColumns(schema.NewIntColumn("a", "int"))},
+			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
+			wantErr: true,
+		},
 	}
 	for i, tt := range tests {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
@@ -865,7 +963,7 @@ func TestPlanChanges(t *testing.T) {
 			}
 			db, _, err := newMigrate(tt.version)
 			require.NoError(t, err)
-			plan, err := db.PlanChanges(context.Background(), "wantPlan", tt.changes)
+			plan, err := db.PlanChanges(context.Background(), "wantPlan", tt.changes, tt.options...)
 			if tt.wantErr {
 				require.Error(t, err, "expect plan to fail")
 				return
@@ -883,6 +981,20 @@ func TestPlanChanges(t *testing.T) {
 	}
 }
 
+func TestDefaultPlan(t *testing.T) {
+	changes, err := DefaultPlan.PlanChanges(context.Background(), "plan", []schema.Change{
+		&schema.AddTable{T: schema.NewTable("t1").SetSchema(schema.New("s1")).AddColumns(schema.NewIntColumn("a", "int"))},
+	})
+	require.NoError(t, err)
+	require.Equal(t, 1, len(changes.Changes))
+	require.Equal(t, "CREATE TABLE `s1`.`t1` (`a` int NOT NULL)", changes.Changes[0].Cmd)
+
+	err = DefaultPlan.ApplyChanges(context.Background(), []schema.Change{
+		&schema.AddTable{T: schema.NewTable("t1").AddColumns(schema.NewIntColumn("a", "int"))},
+	})
+	require.EqualError(t, err, `create "t1" table: cannot execute statements without a database connection. use Open to create a new Driver`)
+}
+
 func newMigrate(version string) (migrate.PlanApplier, *mock, error) {
 	db, m, err := sqlmock.New()
 	if err != nil {
