@@ -16,6 +16,11 @@ import (
 	"ariga.io/atlas/sql/schema"
 )
 
+// DefaultPlan provides basic planning capabilities for PostgreSQL dialects.
+// Note, it is recommended to call Open, create a new Driver and use its
+// migrate.PlanApplier when a database connection is available.
+var DefaultPlan migrate.PlanApplier = &planApply{conn: conn{ExecQuerier: sqlx.NoRows}}
+
 // A planApply provides migration capabilities for schema elements.
 type planApply struct{ conn }
 
@@ -25,7 +30,6 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 		conn: p.conn,
 		Plan: migrate.Plan{
 			Name:          name,
-			Reversible:    true,
 			Transactional: true,
 		},
 		created: make(map[string]*schema.EnumType),
@@ -38,10 +42,8 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 	if err := s.plan(ctx, changes); err != nil {
 		return nil, err
 	}
-	for _, c := range s.Changes {
-		if c.Reverse == "" {
-			s.Reversible = false
-		}
+	if err := sqlx.SetReversible(&s.Plan); err != nil {
+		return nil, err
 	}
 	return &s.Plan, nil
 }
@@ -83,7 +85,7 @@ func (s *state) plan(ctx context.Context, changes []schema.Change) error {
 		case *schema.AddTable:
 			err = s.addTable(ctx, c)
 		case *schema.DropTable:
-			s.dropTable(c)
+			err = s.dropTable(ctx, c)
 		case *schema.ModifyTable:
 			err = s.modifyTable(ctx, c)
 		case *schema.RenameTable:
@@ -105,7 +107,10 @@ func (s *state) topLevel(changes []schema.Change) []schema.Change {
 		switch c := c.(type) {
 		case *schema.AddSchema:
 			b := s.Build("CREATE SCHEMA")
-			if sqlx.Has(c.Extra, &schema.IfNotExists{}) {
+			// Add the 'IF NOT EXISTS' clause if it is explicitly specified, or if the schema name is 'public'.
+			// That is because the 'public' schema is automatically created by PostgreSQL in every new database,
+			// and running the command with this clause will fail in case the schema already exists.
+			if sqlx.Has(c.Extra, &schema.IfNotExists{}) || c.S.Name == "public" {
 				b.P("IF NOT EXISTS")
 			}
 			b.Ident(c.S.Name)
@@ -135,7 +140,7 @@ func (s *state) topLevel(changes []schema.Change) []schema.Change {
 
 // addTable builds and executes the query for creating a table in a schema.
 func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
-	// Create enum types before using them in the `CREATE TABLE` statement.
+	// Create enum types before using them in the 'CREATE TABLE' statement.
 	if err := s.mayAddEnums(ctx, add.T, add.T.Columns...); err != nil {
 		return err
 	}
@@ -194,7 +199,11 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 }
 
 // dropTable builds and executes the query for dropping a table from a schema.
-func (s *state) dropTable(drop *schema.DropTable) {
+func (s *state) dropTable(ctx context.Context, drop *schema.DropTable) error {
+	rs := &state{conn: s.conn, PlanOptions: s.PlanOptions}
+	if err := rs.addTable(ctx, &schema.AddTable{T: drop.T}); err != nil {
+		return fmt.Errorf("calculate reverse for drop table %q: %w", drop.T.Name, err)
+	}
 	b := s.Build("DROP TABLE")
 	if sqlx.Has(drop.Extra, &schema.IfExists{}) {
 		b.P("IF EXISTS")
@@ -204,7 +213,20 @@ func (s *state) dropTable(drop *schema.DropTable) {
 		Cmd:     b.String(),
 		Source:  drop,
 		Comment: fmt.Sprintf("drop %q table", drop.T.Name),
+		// The reverse of 'DROP TABLE' might be a multi
+		// statement operation. e.g., table with indexes.
+		Reverse: func() any {
+			cmd := make([]string, len(rs.Changes))
+			for i, c := range rs.Changes {
+				cmd[i] = c.Cmd
+			}
+			if len(cmd) == 1 {
+				return cmd[0]
+			}
+			return cmd
+		}(),
 	})
+	return nil
 }
 
 // modifyTable builds the statements that bring the table into its modified state.
@@ -324,13 +346,17 @@ func (s *state) modifyTable(ctx context.Context, modify *schema.ModifyTable) err
 			alter = append(alter, change)
 		}
 	}
-	s.dropIndexes(modify.T, dropI...)
+	if err := s.dropIndexes(modify.T, dropI...); err != nil {
+		return err
+	}
 	if len(alter) > 0 {
 		if err := s.alterTable(modify.T, alter); err != nil {
 			return err
 		}
 	}
-	s.addIndexes(modify.T, addI...)
+	if err := s.addIndexes(modify.T, addI...); err != nil {
+		return err
+	}
 	s.append(changes...)
 	return nil
 }
@@ -667,16 +693,19 @@ func (s *state) indexComment(t *schema.Table, idx *schema.Index, to, from string
 	}
 }
 
-func (s *state) dropIndexes(t *schema.Table, indexes ...*schema.Index) {
-	rs := &state{conn: s.conn}
-	rs.addIndexes(t, indexes...)
+func (s *state) dropIndexes(t *schema.Table, indexes ...*schema.Index) error {
+	rs := &state{conn: s.conn, PlanOptions: s.PlanOptions}
+	if err := rs.addIndexes(t, indexes...); err != nil {
+		return err
+	}
 	for i, idx := range indexes {
 		s.append(&migrate.Change{
-			Cmd:     rs.Changes[i].Reverse,
+			Cmd:     rs.Changes[i].Reverse.(string),
 			Comment: fmt.Sprintf("drop index %q from table: %q", idx.Name, t.Name),
 			Reverse: rs.Changes[i].Cmd,
 		})
 	}
+	return nil
 }
 
 func (s *state) mayAddEnums(ctx context.Context, t *schema.Table, columns ...*schema.Column) error {
