@@ -27,6 +27,7 @@ func TestPlanChanges(t *testing.T) {
 	}{
 		{
 			changes: []schema.Change{
+				&schema.AddSchema{S: schema.New("public")},
 				&schema.AddSchema{S: schema.New("test"), Extra: []schema.Clause{&schema.IfNotExists{}}},
 				&schema.DropSchema{S: schema.New("test"), Extra: []schema.Clause{&schema.IfExists{}}},
 				&schema.DropSchema{S: schema.New("test"), Extra: []schema.Clause{}},
@@ -35,6 +36,10 @@ func TestPlanChanges(t *testing.T) {
 				Reversible:    false,
 				Transactional: true,
 				Changes: []*migrate.Change{
+					{
+						Cmd:     `CREATE SCHEMA IF NOT EXISTS "public"`,
+						Reverse: `DROP SCHEMA "public" CASCADE`,
+					},
 					{
 						Cmd:     `CREATE SCHEMA IF NOT EXISTS "test"`,
 						Reverse: `DROP SCHEMA "test" CASCADE`,
@@ -322,13 +327,46 @@ func TestPlanChanges(t *testing.T) {
 		},
 		{
 			changes: []schema.Change{
-				&schema.DropTable{T: &schema.Table{Name: "posts"}},
+				&schema.DropTable{T: schema.NewTable("posts").AddColumns(schema.NewIntColumn("id", "int"))},
 			},
 			wantPlan: &migrate.Plan{
-				Reversible:    false,
+				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
-					{Cmd: `DROP TABLE "posts"`},
+					{
+						Cmd:     `DROP TABLE "posts"`,
+						Reverse: `CREATE TABLE "posts" ("id" integer NOT NULL)`,
+					},
+				},
+			},
+		},
+		{
+			changes: []schema.Change{
+				&schema.DropTable{
+					T: func() *schema.Table {
+						id := schema.NewIntColumn("id", "int")
+						return schema.NewTable("posts").
+							SetComment("a8m's posts").
+							AddColumns(id).
+							AddIndexes(
+								schema.NewIndex("idx").AddColumns(id).SetComment("a8m's index"),
+							)
+					}(),
+				},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd: `DROP TABLE "posts"`,
+						Reverse: []string{
+							`CREATE TABLE "posts" ("id" integer NOT NULL)`,
+							`CREATE INDEX "idx" ON "posts" ("id")`,
+							`COMMENT ON TABLE "posts" IS 'a8m''s posts'`,
+							`COMMENT ON INDEX "idx" IS 'a8m''s index'`,
+						},
+					},
 				},
 			},
 		},
@@ -1195,3 +1233,17 @@ func TestPlanChanges(t *testing.T) {
 		})
 	}
 }
+
+func TestDefaultPlan(t *testing.T) {
+	changes, err := DefaultPlan.PlanChanges(context.Background(), "plan", []schema.Change{
+		&schema.AddTable{T: schema.NewTable("t1").SetSchema(schema.New("s1")).AddColumns(schema.NewIntColumn("a", "int"))},
+	})
+	require.NoError(t, err)
+	require.Equal(t, 1, len(changes.Changes))
+	require.Equal(t, `CREATE TABLE "s1"."t1" ("a" integer NOT NULL)`, changes.Changes[0].Cmd)
+
+	err = DefaultPlan.ApplyChanges(context.Background(), []schema.Change{
+		&schema.AddTable{T: schema.NewTable("t1").AddColumns(schema.NewIntColumn("a", "int"))},
+	})
+	require.EqualError(t, err, `create "t1" table: cannot execute statements without a database connection. use Open to create a new Driver`)
+}
