@@ -42,12 +42,12 @@ func init() {
 
 // evalSpec evaluates an Atlas DDL document into v using the input.
 func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
-	var d doc
-	if err := hclState.Eval(p, &d, input); err != nil {
-		return err
-	}
 	switch v := v.(type) {
 	case *schema.Realm:
+		var d doc
+		if err := hclState.Eval(p, &d, input); err != nil {
+			return err
+		}
 		if err := specutil.Scan(v, d.Schemas, d.Tables, convertTable); err != nil {
 			return fmt.Errorf("specutil: failed converting to *schema.Realm: %w", err)
 		}
@@ -57,6 +57,10 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 			}
 		}
 	case *schema.Schema:
+		var d doc
+		if err := hclState.Eval(p, &d, input); err != nil {
+			return err
+		}
 		if len(d.Schemas) != 1 {
 			return fmt.Errorf("specutil: expecting document to contain a single schema, got %d", len(d.Schemas))
 		}
@@ -68,8 +72,10 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 			return err
 		}
 		*v = *r.Schemas[0]
+	case schema.Schema, schema.Realm:
+		return fmt.Errorf("postgres: Eval expects a pointer: received %[1]T, expected *%[1]T", v)
 	default:
-		return fmt.Errorf("specutil: failed unmarshaling spec. %T is not supported", v)
+		return hclState.Eval(p, v, input)
 	}
 	return nil
 }
@@ -679,6 +685,18 @@ var TypeRegistry = schemahcl.NewRegistry(
 		schemahcl.NewTypeSpec(TypeMoney),
 		schemahcl.NewTypeSpec(TypeTSVector),
 		schemahcl.NewTypeSpec(TypeTSQuery),
+		schemahcl.NewTypeSpec(TypeInt4Range),
+		schemahcl.NewTypeSpec(TypeInt4MultiRange),
+		schemahcl.NewTypeSpec(TypeInt8Range),
+		schemahcl.NewTypeSpec(TypeInt8MultiRange),
+		schemahcl.NewTypeSpec(TypeNumRange),
+		schemahcl.NewTypeSpec(TypeNumMultiRange),
+		schemahcl.NewTypeSpec(TypeTSRange),
+		schemahcl.NewTypeSpec(TypeTSMultiRange),
+		schemahcl.NewTypeSpec(TypeTSTZRange),
+		schemahcl.NewTypeSpec(TypeTSTZMultiRange),
+		schemahcl.NewTypeSpec(TypeDateRange),
+		schemahcl.NewTypeSpec(TypeDateMultiRange),
 		schemahcl.NewTypeSpec("hstore"),
 		schemahcl.NewTypeSpec("sql", schemahcl.WithAttributes(&schemahcl.TypeAttr{Name: "def", Required: true, Kind: reflect.String})),
 	),
