@@ -481,7 +481,7 @@ table "logs" {
 				}
 			}
 		`), &schema.Schema{}, nil)
-		require.EqualError(t, err, "missing attribute logs.partition.type")
+		require.EqualError(t, err, `specutil: cannot convert table "logs": missing attribute logs.partition.type`)
 
 		err = EvalHCLBytes([]byte(`
 			schema "test" {}
@@ -493,7 +493,7 @@ table "logs" {
 				}
 			}
 		`), &schema.Schema{}, nil)
-		require.EqualError(t, err, `missing columns or expressions for logs.partition`)
+		require.EqualError(t, err, `specutil: cannot convert table "logs": missing columns or expressions for logs.partition`)
 
 		err = EvalHCLBytes([]byte(`
 			schema "test" {}
@@ -507,7 +507,7 @@ table "logs" {
 				}
 			}
 		`), &schema.Schema{}, nil)
-		require.EqualError(t, err, `multiple definitions for logs.partition, use "columns" or "by"`)
+		require.EqualError(t, err, `specutil: cannot convert table "logs": multiple definitions for logs.partition, use "columns" or "by"`)
 	})
 }
 
@@ -1357,12 +1357,60 @@ func TestTypes(t *testing.T) {
 		},
 		{
 			typeExpr: "uuid",
-			expected: &UUIDType{T: TypeUUID},
+			expected: &schema.UUIDType{T: TypeUUID},
 		},
 		{
 			typeExpr: "money",
 			expected: &CurrencyType{T: TypeMoney},
 		},
+		{
+			typeExpr: "int4range",
+			expected: &RangeType{T: TypeInt4Range},
+		},
+		{
+			typeExpr: "int4multirange",
+			expected: &RangeType{T: TypeInt4MultiRange},
+		},
+		{
+			typeExpr: "int8range",
+			expected: &RangeType{T: TypeInt8Range},
+		},
+		{
+			typeExpr: "int8multirange",
+			expected: &RangeType{T: TypeInt8MultiRange},
+		},
+		{
+			typeExpr: "numrange",
+			expected: &RangeType{T: TypeNumRange},
+		},
+		{
+			typeExpr: "nummultirange",
+			expected: &RangeType{T: TypeNumMultiRange},
+		},
+		{
+			typeExpr: "tsrange",
+			expected: &RangeType{T: TypeTSRange},
+		},
+		{
+			typeExpr: "tsmultirange",
+			expected: &RangeType{T: TypeTSMultiRange},
+		},
+		{
+			typeExpr: "tstzrange",
+			expected: &RangeType{T: TypeTSTZRange},
+		},
+		{
+			typeExpr: "tstzmultirange",
+			expected: &RangeType{T: TypeTSTZMultiRange},
+		},
+		{
+			typeExpr: "daterange",
+			expected: &RangeType{T: TypeDateRange},
+		},
+		{
+			typeExpr: "datemultirange",
+			expected: &RangeType{T: TypeDateMultiRange},
+		},
 		{
 			typeExpr: `sql("int[]")`,
 			expected: &ArrayType{Type: &schema.IntegerType{T: "int"}, T: "int[]"},
