@@ -186,6 +186,8 @@ type (
 
 // A ChangeKind describes a change kind that can be combined
 // using a set of flags. The zero kind is no change.
+//
+//go:generate stringer -type ChangeKind
 type ChangeKind uint
 
 const (
@@ -199,8 +201,8 @@ const (
 	ChangeAttr ChangeKind = 1 << (iota - 1)
 	// ChangeCharset describes character-set change.
 	ChangeCharset
-	// ChangeCollation describes collation/encoding change.
-	ChangeCollation
+	// ChangeCollate describes collation/encoding change.
+	ChangeCollate
 	// ChangeComment describes comment chang (of any element).
 	ChangeComment
 
