@@ -14,6 +14,11 @@ import (
 	"ariga.io/atlas/sql/schema"
 )
 
+// DefaultPlan provides basic planning capabilities for SQLite dialects.
+// Note, it is recommended to call Open, create a new Driver and use its
+// migrate.PlanApplier when a database connection is available.
+var DefaultPlan migrate.PlanApplier = &planApply{conn: conn{ExecQuerier: sqlx.NoRows}}
+
 // A planApply provides migration capabilities for schema elements.
 type planApply struct{ conn }
 
@@ -23,7 +28,6 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 		conn: p.conn,
 		Plan: migrate.Plan{
 			Name:          name,
-			Reversible:    true,
 			Transactional: true,
 		},
 		PlanOptions: migrate.PlanOptions{
@@ -38,10 +42,16 @@ func (p *planApply) PlanChanges(ctx context.Context, name string, changes []sche
 	if err := s.plan(ctx, changes); err != nil {
 		return nil, err
 	}
-	for _, c := range s.Changes {
-		if c.Reverse == "" {
-			s.Reversible = false
-		}
+	if err := sqlx.SetReversible(&s.Plan); err != nil {
+		return nil, err
+	}
+	// Disable foreign-keys enforcement if it is required
+	// by one of the changes in the plan.
+	if s.skipFKs {
+		// Callers should note that these 2 pragmas are no-op in transactions,
+		// See: https://sqlite.org/pragma.html#pragma_foreign_keys.
+		s.Changes = append([]*migrate.Change{{Cmd: "PRAGMA foreign_keys = off", Comment: "disable the enforcement of foreign-keys constraints"}}, s.Changes...)
+		s.append(&migrate.Change{Cmd: "PRAGMA foreign_keys = on", Comment: "enable back the enforcement of foreign-keys constraints"})
 	}
 	return &s.Plan, nil
 }
@@ -71,7 +81,7 @@ func (s *state) plan(ctx context.Context, changes []schema.Change) (err error) {
 		case *schema.AddTable:
 			err = s.addTable(ctx, c)
 		case *schema.DropTable:
-			err = s.dropTable(c)
+			err = s.dropTable(ctx, c)
 		case *schema.ModifyTable:
 			err = s.modifyTable(ctx, c)
 		case *schema.RenameTable:
@@ -83,14 +93,6 @@ func (s *state) plan(ctx context.Context, changes []schema.Change) (err error) {
 			return err
 		}
 	}
-	// Disable foreign-keys enforcement if it is required
-	// by one of the changes in the plan.
-	if s.skipFKs && s.conn.fkEnabled {
-		// Callers should note that these 2 pragmas are no-op in transactions,
-		// and therefore, should not call BEGIN manually. https://sqlite.org/pragma.html#pragma_foreign_keys
-		s.Changes = append([]*migrate.Change{{Cmd: "PRAGMA foreign_keys = off", Comment: "disable the enforcement of foreign-keys constraints"}}, s.Changes...)
-		s.append(&migrate.Change{Cmd: "PRAGMA foreign_keys = on", Comment: "enable back the enforcement of foreign-keys constraints"})
-	}
 	return nil
 }
 
@@ -144,7 +146,11 @@ func (s *state) addTable(ctx context.Context, add *schema.AddTable) error {
 }
 
 // dropTable builds and executes the query for dropping a table from a schema.
-func (s *state) dropTable(drop *schema.DropTable) error {
+func (s *state) dropTable(ctx context.Context, drop *schema.DropTable) error {
+	rs := &state{conn: s.conn, PlanOptions: s.PlanOptions}
+	if err := rs.addTable(ctx, &schema.AddTable{T: drop.T}); err != nil {
+		return fmt.Errorf("calculate reverse for drop table %q: %w", drop.T.Name, err)
+	}
 	s.skipFKs = true
 	b := s.Build("DROP TABLE").Ident(drop.T.Name)
 	if sqlx.Has(drop.Extra, &schema.IfExists{}) {
@@ -154,6 +160,18 @@ func (s *state) dropTable(drop *schema.DropTable) error {
 		Cmd:     b.String(),
 		Source:  drop,
 		Comment: fmt.Sprintf("drop %q table", drop.T.Name),
+		// The reverse of 'DROP TABLE' might be a multi
+		// statement operation. e.g., table with indexes.
+		Reverse: func() any {
+			cmd := make([]string, len(rs.Changes))
+			for i, c := range rs.Changes {
+				cmd[i] = c.Cmd
+			}
+			if len(cmd) == 1 {
+				return cmd[0]
+			}
+			return cmd
+		}(),
 	})
 	return nil
 }
@@ -244,7 +262,7 @@ func (s *state) dropIndexes(t *schema.Table, indexes ...*schema.Index) error {
 	}
 	for i := range rs.Changes {
 		s.append(&migrate.Change{
-			Cmd:     rs.Changes[i].Reverse,
+			Cmd:     rs.Changes[i].Reverse.(string),
 			Reverse: rs.Changes[i].Cmd,
 			Comment: fmt.Sprintf("drop index %q from table: %q", indexes[i].Name, t.Name),
 		})
@@ -461,7 +479,7 @@ func (s *state) alterTable(modify *schema.ModifyTable) error {
 func (s *state) tableSeq(ctx context.Context, add *schema.AddTable) error {
 	var inc AutoIncrement
 	switch pk := add.T.PrimaryKey; {
-	// Sequence was set on the table.
+	// Sequence was set on table attributes.
 	case sqlx.Has(add.T.Attrs, &inc) && inc.Seq > 0:
 	// Sequence was set on table primary-key (a single column PK).
 	case pk != nil && len(pk.Parts) == 1 && pk.Parts[0].C != nil && sqlx.Has(pk.Parts[0].C.Attrs, &inc) && inc.Seq > 0:
